@@ -145,6 +145,10 @@ bool XAP_UnixFontHandle::doesGlyphExist(UT_UCS4Char g)
 	return m_font->doesGlyphExist(g);
 }
 
+static float fixedToFloat(signed long in)
+{
+  float res = static_cast<float>(in*64);
+}
 //
 // UT_Rect of glyph in Logical units.
 // rec.left = bearing Left (distance from origin to start)
@@ -154,25 +158,38 @@ bool XAP_UnixFontHandle::doesGlyphExist(UT_UCS4Char g)
 //
 bool  XAP_UnixFontHandle::glyphBox(UT_UCS4Char g, UT_Rect & rec, GR_Graphics * pG) 
 {
+	UT_return_val_if_fail( pG, false );
+	
 	XftFaceLocker locker(m_font->getLayoutXftFont(GR_CharWidthsCache::CACHE_FONT_SIZE));
+
 	FT_Face pFace = locker.getFace();
 
 	UT_UCS4Char cChar = g;
-	if(m_font->isSymbol()) cChar = static_cast<UT_UCS4Char>(adobeToUnicode(g));
+	
+	if(m_font->isSymbol())
+		cChar = static_cast<UT_UCS4Char>(adobeToUnicode(g));
+	
 	FT_UInt glyph_index = FT_Get_Char_Index(pFace, cChar);
-	FT_Error error =
-		FT_Load_Glyph(pFace, glyph_index,
-					FT_LOAD_LINEAR_DESIGN |
-					FT_LOAD_IGNORE_TRANSFORM |
-					FT_LOAD_NO_BITMAP | FT_LOAD_NO_SCALE);
-	if (error) {
+
+	FT_Error error = FT_Load_Glyph(pFace, glyph_index,
+								   FT_LOAD_LINEAR_DESIGN |
+								   FT_LOAD_IGNORE_TRANSFORM |
+								   FT_LOAD_NO_BITMAP |
+								   FT_LOAD_NO_SCALE);
+	if (error)
+	{
 		return false;
 	}
-	rec.left = pFace->glyph->metrics.horiBearingX;
-	rec.width = pFace->glyph->metrics.width;
-	rec.top = pFace->glyph->metrics.horiBearingY;
-	rec.height = pFace->glyph->metrics.height;
-	xxx_UT_DEBUGMSG((" left %d width %d top %d height %d \n",rec.left,rec.width,rec.top,rec.height));
+
+	UT_uint32 iSize = m_size * pG->getResolution() / pG->getDeviceResolution();
+	
+	rec.left   = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.horiBearingX));
+	rec.width  = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.width));
+	rec.top    = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.horiBearingY));
+	rec.height = static_cast<UT_sint32>(fontPoints2float(iSize, pFace, pFace->glyph->metrics.height));
+	
+	UT_DEBUGMSG((" left %d width %d top %d height %d \n",rec.left,rec.width,rec.top,rec.height));
+
 	return true;
 }
 /*******************************************************************/
@@ -789,7 +806,7 @@ XftFont *XAP_UnixFont::getLayoutXftFont(UT_uint32 pixelsize) const
 // returns different fonts depending on the zoom percentage
 XftFont *XAP_UnixFont::getDeviceXftFont(UT_uint32 pixelsize, UT_uint32 zoomPercentage) const
 {
-#define extra_font_size 1.5
+#define extra_font_size 0.5 // was 1.5
 	XftFont *pXftFont = getFontFromCache(static_cast<UT_sint32>(extra_font_size + static_cast<double>(pixelsize*zoomPercentage)/static_cast<double>(100.)) , false, zoomPercentage);
 	if (pXftFont)
 		return pXftFont;
