@@ -108,6 +108,10 @@
 #include "xap_UnixDialogHelper.h"
 //#include <gtk/gtk.h>
 
+#ifdef ENABLE_BINRELOC
+#include "prefix.h"
+#endif // ENABLE_BINRELOC
+
 
 #ifdef GTK_WIN_POS_CENTER_ALWAYS
 #define WIN_POS GTK_WIN_POS_CENTER_ALWAYS
@@ -667,6 +671,20 @@ void AP_UnixApp::pasteFromClipboard(PD_DocumentRange * pDocRange, bool bUseClipb
 	}
     else if (AP_UnixClipboard::isImageTag(szFormatFound))
       {
+		  UT_DEBUGMSG(("Format Found = %s \n",szFormatFound));
+		  if(strncmp(szFormatFound,"application",11) == 0) // embedded object
+		  {
+			  IE_Imp * pImp = NULL;
+			  IEGraphicFileType iegft = IE_Imp::fileTypeForContents(reinterpret_cast<char *>(const_cast<unsigned char *>(pData)),iLen);
+			  IE_Imp::constructImporter(pDocRange->m_pDoc,NULL,iegft,&pImp);
+			  if(pImp == NULL)
+			  {
+					  goto retry_text;
+			  }
+			  bool b = pImp->pasteFromBuffer(pDocRange,pData,iLen);
+			  DELETEP(pImp);
+			  return;
+		  }
 		  IE_ImpGraphic * pIEG = NULL;
 		  FG_Graphic * pFG = NULL;
 		  IEGraphicFileType iegft = IEGFT_Unknown;
@@ -735,12 +753,7 @@ bool AP_UnixApp::canPasteFromClipboard(void)
 extern "C" {
 
 	// return > 0 for directory entries ending in ".so" ".sl" and the like
-#if defined (__APPLE__) || defined (__FreeBSD__) || defined (__OpenBSD__) \
-	|| defined(_AIX) || defined(__sgi)
-	static int so_only (struct dirent *d)
-#else
-	static int so_only (const struct dirent *d)
-#endif
+	static int so_only (ABI_SCANDIR_SELECT_QUALIFIER struct dirent *d)
 	{
 		const char * name = d->d_name;
 		
@@ -777,7 +790,19 @@ void AP_UnixApp::loadAllPlugins ()
   UT_String pluginDir;
 
   // the global plugin directory
-  pluginDir = ABIWORD_PLUGINDIR;
+#ifdef ENABLE_BINRELOC
+  pluginDir = LIBDIR;
+#else
+  pluginDir = ABIWORD_LIBDIR;
+#endif
+  pluginDir += "/";
+  pluginDir += ABIWORD_APP_NAME;
+  pluginDir += "-";
+  pluginDir += PACKAGE_VERSION;
+  pluginDir += "/plugins/";
+
+  UT_DEBUGMSG(("pluginDir: '%s'\n", pluginDir.c_str ()));
+
   pluginList[0] = pluginDir;
 
   // the user-local plugin directory
@@ -1269,7 +1294,8 @@ GR_Image * AP_UnixApp::_showSplash(UT_uint32 delay)
 			}
 		}
 		// create a centered window the size of our image
-		wSplash = gtk_window_new(GTK_WINDOW_POPUP); //GTK_WINDOW_DIALOG
+		wSplash = gtk_window_new(GTK_WINDOW_TOPLEVEL); //GTK_WINDOW_DIALOG
+		gtk_window_set_decorated (GTK_WINDOW(wSplash), FALSE);
 		gtk_window_set_default_size (GTK_WINDOW (wSplash),
 									 iSplashWidth, iSplashHeight);
 		gtk_window_set_resizable(GTK_WINDOW(wSplash), FALSE);
@@ -1507,6 +1533,9 @@ int AP_UnixApp::main(const char * szAppName, int argc, const char ** argv)
 			UT_DEBUGMSG(("DOM: not parsing command line or showing app\n"));
 		}
 	}
+	else {
+		UT_DEBUGMSG(("No DISPLAY: this may not be what you want.\n"));
+	}
 
 	// Step 4: Destroy the App.  It should take care of deleting all frames.
 	pMyUnixApp->shutdown();
