@@ -52,7 +52,6 @@
 #include "xad_Document.h"
 #include "xap_Module.h"
 #include "ap_App.h"
-#include "ie_imp_MathML.h"
 #include "ie_imp.h"
 #include "ie_impGraphic.h"
 #include "ut_assert.h"
@@ -91,36 +90,9 @@
 #include "ut_files.h"
 #endif
 
-static const AbiMathViewEntityMapItem mathmlEntitiesMap[] = {
-#include "entitiesMap.inc"
-};
+#include "itex2MML.h"
 
-static int compareEntities(const void * pSz, const void * pEnt)
-{
-  const char * sz1 =NULL;
-  const char * sz2 = NULL;
-  AbiMathViewEntityMapItem ** pE =NULL;
-  sz1 = reinterpret_cast<const char*>(pSz);
-  pE = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt)));
-  sz2 = (*pE)->szEntity;
-  return UT_strcmp(sz1,sz2);
-}
-
-
-static int sortEntities(const void * pEnt1, const void * pEnt2)
-{
-	const char * sz1 = NULL;
-	const char * sz2 = NULL;
-	AbiMathViewEntityMapItem ** pE1 = NULL;
-	AbiMathViewEntityMapItem ** pE2 = NULL;
-
-	pE1 = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt1)));
-	pE2 = reinterpret_cast<AbiMathViewEntityMapItem **>(reinterpret_cast<void ** >(const_cast<void *>(pEnt2)));
-	sz1 = (*pE1)->szEntity;
-	sz2 = (*pE2)->szEntity;
-
-	return UT_strcmp(sz1,sz2);
-}
+static GR_MathManager * pMathManager = NULL; // single plug-in instance of GR_MathManager
 
 //
 // AbiMathView_addToMenus
@@ -467,7 +439,7 @@ AbiMathView_FileInsert(AV_View* v, EV_EditMethodCallData *d)
 
 	UT_DEBUGMSG(("fileInsertMathML: loading [%s]\n",sNewFile.utf8_str()));
    
-	IE_Imp_MathML * pImpMathML = new IE_Imp_MathML(pDoc);
+	IE_Imp_MathML * pImpMathML = new IE_Imp_MathML(pDoc, pMathManager->EntityTable());
 	UT_Error errorCode = pImpMathML->importFile(sNewFile.utf8_str());
 
 	if (errorCode != UT_OK)
@@ -548,7 +520,6 @@ GR_MathManager::GR_MathManager(GR_Graphics* pG) :
 {
 	m_vecMathView.clear();
 	m_vecItems.clear();
-	m_vecEntityMap;
 }
 
 GR_MathManager::~GR_MathManager()
@@ -575,23 +546,29 @@ void GR_MathManager::initialize(void)
 	// load the gtkmathview settings and operator dictionaries from the private user directory, ...
 	UT_UTF8String userConfDir(pApp->getUserPrivateDirectory());
 	UT_UTF8String userDictDir(pApp->getUserPrivateDirectory());
+	UT_UTF8String userLocalDictDir(pApp->getUserPrivateDirectory());
 #if defined(WIN32)
 	userConfDir += "\\math\\gtkmathview.conf.xml";
 	userDictDir += "\\math\\dictionary.xml";
+	userLocalDictDir += "\\math\\dictionary-local.xml";
 #else
 	userConfDir += "/math/gtkmathview.conf.xml";
 	userDictDir += "/math/dictionary.xml";
+	userLocalDictDir += "/math/dictionary-local.xml";
 #endif
 
 	// ... or from the (common) AbiSuite directory
 	UT_UTF8String libConfDir(pApp->getAbiSuiteLibDir());
 	UT_UTF8String libDictDir(pApp->getAbiSuiteLibDir());
+	UT_UTF8String libLocalDictDir(pApp->getAbiSuiteLibDir());
 #if defined(WIN32)	 
 	libConfDir += "\\math\\gtkmathview.conf.xml";
 	libDictDir += "\\math\\dictionary.xml";
+	libLocalDictDir += "\\math\\dictionary-local.xml";
 #else
 	libConfDir += "/math/gtkmathview.conf.xml";
 	libDictDir += "/math/dictionary.xml";
+	libLocalDictDir += "/math/dictionary-local.xml";
 #endif
 
 	// add the configuration paths
@@ -613,34 +590,28 @@ void GR_MathManager::initialize(void)
 	logger->setLogLevel(LOG_INFO);
 	 
 	// add the dictionary paths
-	configuration->add("dictionary/path", userDictDir.utf8_str());
-	configuration->add("dictionary/path", libDictDir.utf8_str());
 
 #ifdef XP_TARGET_COCOA
 	if (const char * resources = getenv("ABIWORD_COCOA_BUNDLED_RESOURCES"))
 	{
 		UT_UTF8String bundleDictDir(resources);
+		UT_UTF8String bundleLocalDictDir(resources);
 		bundleDictDir += "/gtkmathview/dictionary.xml";
+		bundleLocalDictDir += "/gtkmathview/dictionary-local.xml";
 		configuration->add("dictionary/path", bundleDictDir.utf8_str());
+		configuration->add("dictionary/path", bundleLocalDictDir.utf8_str());
 	}
 #endif
+	configuration->add("dictionary/path", libDictDir.utf8_str());
+	configuration->add("dictionary/path", libLocalDictDir.utf8_str());
+	configuration->add("dictionary/path", userDictDir.utf8_str());
+	configuration->add("dictionary/path", userLocalDictDir.utf8_str());
 
 	SmartPtr<GR_Abi_MathGraphicDevice> mathGraphicDevice 
 			= GR_Abi_MathGraphicDevice::create(logger, configuration, getGraphics());
 	m_pMathGraphicDevice = mathGraphicDevice;
 	m_pAbiContext = new GR_Abi_RenderingContext(getGraphics());
 	m_pOperatorDictionary = initOperatorDictionary<libxml2_MathView>(logger, configuration);
-
-	UT_sint32 i = 0;
-	AbiMathViewEntityMapItem * pEnt;
-	AbiMathViewEntityMapItem mi;
-	UT_sint32 MapSize = sizeof(mathmlEntitiesMap)/(sizeof(mi));
-	for (i=0; i<MapSize-1;i++)
-	{
-		pEnt = const_cast<AbiMathViewEntityMapItem * >(&mathmlEntitiesMap[i]);
-		m_vecEntityMap.addItem(pEnt);
-	}
-	m_vecEntityMap.qsort(sortEntities);
 }
 
 UT_sint32  GR_MathManager::_makeMathView(void)
@@ -871,12 +842,6 @@ void GR_MathManager::releaseEmbedView(UT_sint32 uid)
 	pMathView->resetRootElement();
 }
 
-extern "C"
-{
-	extern void itex2MML_yyrestart(FILE *input_file);
-	extern int  itex2MML_yyparse(char **str);
-}
-
 bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf & To)
 {
 	XAP_App * pApp = XAP_App::getApp();
@@ -894,20 +859,9 @@ bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf &
 	sLatex.appendBuf(From, myWC);
 	sLatex += "\\]";
 
-	/* TODO: rewrite itex2MML to be able to use the From buffer directly
-		 instread of through a FILE* */
-
-	FILE *tmp = tmpfile();
-	fwrite(sLatex.utf8_str(), sLatex.size(), 1, tmp);
-	fflush(tmp);
-	rewind(tmp);
-	itex2MML_yyrestart(tmp);
-
-	char *mathml = 0;
-	int res = itex2MML_yyparse(&mathml);
-	fclose(tmp);
+	char * mathml = itex2MML_parse(sLatex.utf8_str(), sLatex.size());
 	
-	if (res != 0)
+	if (!mathml)
 	{
 		pFrame->showMessageBox("itex2MML failed to convert the LaTeX equation into MathML, sorry!\n", // TODO: fix message
 			XAP_Dialog_MessageBox::b_O, 
@@ -916,7 +870,7 @@ bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf &
 	}
 
 	UT_UTF8String sMathML(mathml);
-	FREEP(mathml);
+	itex2MML_free_string(mathml);
 
 	if (sMathML.size() == 0)
 	{
@@ -930,94 +884,12 @@ bool GR_MathManager::convert(UT_uint32 iConType, UT_ByteBuf & From, UT_ByteBuf &
 		return false;
 	}
 
-	char * szRawChars = NULL;
-	szRawChars = const_cast<char *>(sMathML.utf8_str());
-	UT_DEBUGMSG(("Input MathML %s \n", szRawChars));
+	UT_DEBUGMSG(("Input MathML %s \n", sMathML.utf8_str()));
 
-	char * psz0 = szRawChars;
-	char * psz1 = szRawChars;
-	char * psz2 = NULL;
-	UT_UTF8String sLeft;
-	bool bNew = false;
-	bool bLoop = true;
-	while (bLoop)
-	{
-		UT_DEBUGMSG(("Remaining string %s \n",psz0));
-
-		psz1 = strstr(psz0,"&");
-		if (psz1 == NULL)
-		{
-			bLoop = false;
-			break;
-		}
-
-		psz2 = strstr(psz1,";");
-		if(psz2 == NULL)
-		{
-			bLoop = false;
-			break;
-		}
-
-		//
-		// Make a copy of the string
-		//
-		UT_sint32 isize = static_cast<UT_sint32>(psz2-psz1);
-		char * pszNew = new char[isize];
-		pszNew[isize-1] = 0;
-		UT_sint32 i = 0;
-		for(i=0; i< (isize -1);i++)
-		{
-			pszNew[i] = psz1[1+i];
-		}
-		UT_DEBUGMSG(("Extracted entity %s \n", pszNew));
-
-		char * szEntVal = NULL;
-		UT_sint32 pos = m_vecEntityMap.binarysearch(reinterpret_cast<void *>(pszNew),compareEntities);
-		DELETEPV(pszNew);
-
-		if(pos >= 0)
-		{
-			//
-			// OK slice up the string and but in the entity value
-			//
-			bNew = true;
-			AbiMathViewEntityMapItem * pE = m_vecEntityMap.getNthItem(pos);
-			szEntVal = pE->szVal;
-			*psz1 = 0; // rub out the '&'
-			sLeft += psz0;
-			sLeft += szEntVal;
-			*psz2= 0;
-			psz1 = psz2;
-			psz1++;
-			psz0 = psz1;
-		}
-		else
-		{
-			UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
-			bNew = false;
-			break;
-		}
-
-		if(psz0 >= &szRawChars[sMathML.size()-1])
-		{
-			bLoop = false;
-			break;
-		}
-	}
-
-	if (bNew)
-	{
-		sLeft += psz0;
-		sMathML = sLeft;
-	}
-	UT_DEBUGMSG(("MATHML Inserted %s \n",sMathML.utf8_str()));
-	To.ins(0,reinterpret_cast<const UT_Byte *>(sMathML.utf8_str()),static_cast<UT_uint32>(sMathML.size()));
-
-	return true;
+	return EntityTable().convert(sMathML.utf8_str(), sMathML.size(), To);
 }
 
 static UT_uint32 MathManagerUID = 0; 
-static GR_MathManager  *pMathManager = NULL;
 
 ABI_PLUGIN_DECLARE(AbiMathView)
 
