@@ -339,13 +339,17 @@ OD_Style_Style* OD_Office_Styles::addDefaultStyle(const XML_Char** ppAtts,
 void OD_Office_Styles::_linkStyles() {
     
     _linkStyles(m_textStyleStyles);
-    _linkParagraphStyles(false);
-    _linkStyles(m_sectionStyleStyles);
+    _linkStyles(m_textStyleStyles_contentStream, &m_textStyleStyles);    
     
-    _linkStyles(m_textStyleStyles_contentStream, &m_textStyleStyles);
+    _linkParagraphStyles(false);
     _linkParagraphStyles(true);
+        
+    _linkStyles(m_sectionStyleStyles);
     _linkStyles(m_sectionStyleStyles_contentStream, &m_sectionStyleStyles);
     
+    _linkStyles(m_graphicStyleStyles);
+    _linkStyles(m_graphicStyleStyles_contentStream, &m_graphicStyleStyles);
+    
     _linkMasterStyles();
     _linkListStyles();
 }
