@@ -42,19 +42,28 @@
 OD_StreamListener::OD_StreamListener(PD_Document* pAbiDocument,
                                      GsfInfile* pGsfInfile,
                                      OD_Office_Styles* pStyles,
-                                     OD_Abi_Data& rAbiData)
+                                     OD_Abi_Data& rAbiData,
+                                     OD_ElementStack* pElementStack)
                                      :
                                      m_pAbiDocument(pAbiDocument),
                                      m_pGsfInfile(pGsfInfile),
                                      m_pStyles(pStyles),
                                      m_rAbiData(rAbiData),
                                      m_pCurrentState(NULL),
-                                     m_deleteCurrentWhenPop(false),
-                                     m_bResumingPostponedParsing(false)
+                                     m_deleteCurrentWhenPop(false)
 {
     UT_ASSERT(m_pAbiDocument);
     UT_ASSERT(m_pGsfInfile);
     UT_ASSERT(m_pStyles);
+
+
+    // This is done for supporting nested StreamListeners, used when we are
+    // resuming postponed elements.    
+    if (pElementStack == NULL) {
+        m_pElementStack = new OD_ElementStack();
+    } else {
+        m_pElementStack = pElementStack;
+    }
 }
 
 
@@ -74,8 +83,9 @@ OD_StreamListener::~OD_StreamListener()
 /**
  * 
  */
-void OD_StreamListener::startElement (const XML_Char* pName,
-                                      const XML_Char** ppAtts)
+void OD_StreamListener::_startElement (const XML_Char* pName,
+                                      const XML_Char** ppAtts,
+                                      bool doingRecursion)
 {
     UT_ASSERT(m_pCurrentState);
 
@@ -83,18 +93,29 @@ void OD_StreamListener::startElement (const XML_Char* pName,
     m_pCurrentState->startElement(pName, ppAtts, m_stateAction);
     
     if (m_stateAction.getAction() != m_stateAction.ACTION_NONE) {
+        OD_ListenerState* pState;
+        
+        pState = m_pCurrentState;
         _handleStateAction();
-        m_pCurrentState->startElement(pName, ppAtts, m_stateAction);
+        
+        if (m_pCurrentState != NULL && pState != m_pCurrentState) {
+            // The state has changed.
+            this->_startElement(pName, ppAtts, true);
+        }
     }
     
-    m_elementStack.startElement(pName, ppAtts);
+    // A check to avoid calling it more than once for a single, actual, XML
+    // start element tag.
+    if (!doingRecursion) {
+        m_pElementStack->startElement(pName, ppAtts);
+    }
 }
 
 
 /**
  * 
  */
-void OD_StreamListener::endElement (const XML_Char* pName)
+void OD_StreamListener::_endElement (const XML_Char* pName, bool doingRecursion)
 {
     UT_ASSERT(m_pCurrentState);
     
@@ -102,15 +123,21 @@ void OD_StreamListener::endElement (const XML_Char* pName)
     m_pCurrentState->endElement(pName, m_stateAction);
     
     if (m_stateAction.getAction() != m_stateAction.ACTION_NONE) {
+        OD_ListenerState* pState;
+
+        pState = m_pCurrentState;
         _handleStateAction();
-        if (m_pCurrentState) {
-            m_pCurrentState->endElement(pName, m_stateAction);
+        
+        if (m_pCurrentState != NULL && pState != m_pCurrentState) {
+            // The state has changed.
+            this->_endElement(pName, true);
         }
     }
     
-	m_elementStack.endElement(pName);
-    if (m_pCurrentState == NULL) {
-        UT_ASSERT(m_elementStack.isEmpty());
+    // A check to avoid calling it more than once for a single, actual, XML
+    // start element tag.    
+    if (!doingRecursion) {
+    	m_pElementStack->endElement(pName);
     }
 }
 
@@ -149,6 +176,20 @@ UT_Error OD_StreamListener::setState(const char* pStateName)
 }
 
 
+/**
+ * 
+ */
+void OD_StreamListener::setState(OD_ListenerState* pState, bool deleteWhenPop) {
+    
+    UT_ASSERT(m_stateStack.getItemCount() == 0);
+    UT_ASSERT(m_pCurrentState == NULL);
+    _clear();
+    
+    m_pCurrentState = pState;
+    m_deleteCurrentWhenPop = deleteWhenPop;
+}
+
+
 /**
  * Push or pop the stack according to the action stated by the current state.
  */
@@ -207,7 +248,7 @@ void OD_StreamListener::_handleStateAction ()
                 pPostponeState = new OD_Postpone_ListenerState(
                                                   m_stateAction.getState(),
                                                   m_stateAction.getDeleteWhenPop(),
-                                                  m_elementStack);
+                                                  *m_pElementStack);
             } else {
                 OD_ListenerState* pNewState;
                 
@@ -218,7 +259,7 @@ void OD_StreamListener::_handleStateAction ()
                 pPostponeState = new OD_Postpone_ListenerState(
                                                   pNewState,
                                                   m_stateAction.getDeleteWhenPop(),
-                                                  m_elementStack);
+                                                  *m_pElementStack);
             }
             m_postponedParsing.addItem(pPostponeState);
             
@@ -234,8 +275,7 @@ void OD_StreamListener::_handleStateAction ()
             
         case OD_ListenerStateAction::ACTION_BRINGUPALL:
             
-            if (!m_bResumingPostponedParsing) {
-        
+            {
                 UT_uint32 i;
                 bool comeBackAfter = m_stateAction.getComeBackAfter();
                             
@@ -250,15 +290,14 @@ void OD_StreamListener::_handleStateAction ()
                     m_stateAction.popState();
                     this->_handleStateAction();
                 }
-            
             }
+            
             break;
         
             
         case OD_ListenerStateAction::ACTION_BRINGUP:
         
-            if (!m_bResumingPostponedParsing &&
-                    m_postponedParsing.getItemCount() > 0) {
+            if (m_postponedParsing.getItemCount() > 0) {
                     
                 OD_Postpone_ListenerState* pPostponedState;
                 
@@ -269,9 +308,17 @@ void OD_StreamListener::_handleStateAction ()
                     pPostponedState->getParserState()->getStateName();
                     
                 if (rStateName == m_stateAction.getStateName()) {
+                    
+                    bool comeBackAfter = m_stateAction.getComeBackAfter();
+                    
                     _resumeParsing(pPostponedState);
                     DELETEP(pPostponedState);
                     m_postponedParsing.pop_back();
+                    
+                    if (!comeBackAfter) {
+                        m_stateAction.popState();
+                        this->_handleStateAction();
+                    }
                 }
             }
             break;
@@ -299,8 +346,6 @@ void OD_StreamListener::_clear ()
         }
     }
     m_stateStack.clear();
-    
-    m_elementStack.clear();
 }
 
 
@@ -316,30 +361,30 @@ OD_ListenerState* OD_StreamListener::_createState(const char* pStateName) {
     if (!UT_strcmp("StylesStream", pStateName)) {
         
         pState = new OD_StylesStream_ListenerState(m_pAbiDocument, m_pGsfInfile,
-                                                   m_pStyles, m_elementStack);
+                                                   m_pStyles, *m_pElementStack);
         
     } else if (!UT_strcmp("MetaStream", pStateName)) {
         
-        pState = new OD_MetaStream_ListenerState(m_pAbiDocument, m_elementStack);
+        pState = new OD_MetaStream_ListenerState(m_pAbiDocument, *m_pElementStack);
         
     } else if (!UT_strcmp("SettingsStream", pStateName)) {
         
-        pState = new OD_SettingsStream_ListenerState(m_elementStack);
+        pState = new OD_SettingsStream_ListenerState(*m_pElementStack);
         
     } else if (!UT_strcmp("ContentStream", pStateName)) {
         
         pState = new OD_ContentStream_ListenerState(m_pAbiDocument, m_pGsfInfile,
-                                                    m_pStyles, m_elementStack);
+                                                    m_pStyles, *m_pElementStack);
 
     } else if (!UT_strcmp("TextContent", pStateName)) {
         
         pState = new OD_TextContent_ListenerState(m_pAbiDocument, m_pStyles,
-                                                  m_elementStack);
+                                                  *m_pElementStack);
     } else if (!UT_strcmp("Frame", pStateName)) {
         
         pState = new OD_Frame_ListenerState(m_pAbiDocument, m_pStyles,
                                             m_rAbiData,
-                                            m_elementStack);
+                                            *m_pElementStack);
     }
     
     return pState;
@@ -356,14 +401,14 @@ void OD_StreamListener::_resumeParsing(OD_Postpone_ListenerState* pPostponeState
     OD_Postpone_ListenerState::EndElementCall* pEndCall = NULL;
     OD_Postpone_ListenerState::CharDataCall* pCharDataCall = NULL;
     
-    m_bResumingPostponedParsing = true;
-    
-    m_stateStack.push_back(
-        OD_StreamListener::StackCell(m_pCurrentState, m_deleteCurrentWhenPop));
-    
-    m_pCurrentState = pPostponeState->getParserState();
-    m_deleteCurrentWhenPop = pPostponeState->getDeleteParserStateWhenPop();
+    OD_StreamListener streamListener(m_pAbiDocument, m_pGsfInfile,
+                                     m_pStyles, m_rAbiData,
+                                     m_pElementStack);
+                                     
+    streamListener.setState(pPostponeState->getParserState(),
+                            pPostponeState->getDeleteParserStateWhenPop());
     
+
     count = pPostponeState->m_XMLCalls.getItemCount();
     for (i=0; i<count; i++) {
         switch ( (pPostponeState->m_XMLCalls[i])->getType() ) {
@@ -371,44 +416,27 @@ void OD_StreamListener::_resumeParsing(OD_Postpone_ListenerState* pPostponeState
             case OD_Postpone_ListenerState::XMLCallType_StartElement:
                 pStartCall = (OD_Postpone_ListenerState::StartElementCall*)
                                 pPostponeState->m_XMLCalls[i];
-                this->startElement(pStartCall->m_pName,
+                                
+                streamListener.startElement(
+                                   pStartCall->m_pName,
                                    (const XML_Char**) pStartCall->m_ppAtts);
                 break;
                 
-                
             case OD_Postpone_ListenerState::XMLCallType_EndElement:
                 pEndCall = (OD_Postpone_ListenerState::EndElementCall*)
                                 pPostponeState->m_XMLCalls[i];
-                this->endElement(pEndCall->m_pName);
+                                
+                streamListener.endElement(pEndCall->m_pName);
                 break;
                 
             case OD_Postpone_ListenerState::XMLCallType_CharData:
                 pCharDataCall = (OD_Postpone_ListenerState::CharDataCall*)
                                 pPostponeState->m_XMLCalls[i];
-                this->charData(pCharDataCall->m_pBuffer, pCharDataCall->m_length);
+                                
+                streamListener.charData(pCharDataCall->m_pBuffer,
+                                         pCharDataCall->m_length);
                 break;
         }
     }
     
-    
-    // Return to the original state (if not there already).
-    if (pOriginalState != m_pCurrentState) {
-        OD_StreamListener::StackCell stackCell;
-            
-        stackCell = m_stateStack.getLastItem();
-        
-        if (m_deleteCurrentWhenPop) {
-            DELETEP(m_pCurrentState);
-        }
-        
-        m_pCurrentState = stackCell.m_pState;
-        m_deleteCurrentWhenPop = stackCell.m_deleteWhenPop;
-        
-        m_stateStack.pop_back();
-        
-        UT_ASSERT(pOriginalState == m_pCurrentState);
-    }
-    
-    // We're done.
-    m_bResumingPostponedParsing = false;
 }
