@@ -58,7 +58,8 @@ OD_TextContent_ListenerState::OD_TextContent_ListenerState (
                   m_bOnContentStream(false),
                   m_pCurrentListStyle(NULL),
                   m_listLevel(0),
-                  m_alreadyDefinedAbiParagraphForList(false)
+                  m_alreadyDefinedAbiParagraphForList(false),
+                  m_elementParsingLevel(0)
 {
     UT_ASSERT(m_pAbiDocument);
     UT_ASSERT(m_pStyles);
@@ -287,12 +288,16 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
                !UT_strcmp(pName, "style:footer") ||
                !UT_strcmp(pName, "style:header-left") ||
                !UT_strcmp(pName, "style:footer-left")) {
+                
+        UT_ASSERT(m_elementParsingLevel == 0);
+                
         // We are inside a header/footer so, there is already a section defined
         // on the AbiWord document.
         m_bInSection = true;
         m_bOnContentStream = false;
         
     } else if (!UT_strcmp(pName, "office:text")) {
+        UT_ASSERT(m_elementParsingLevel == 0);
         m_bOnContentStream = true;
         
     } else if (!UT_strcmp(pName, "text:list")) {
@@ -316,6 +321,7 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
         m_alreadyDefinedAbiParagraphForList = false;
         
     } else if (!UT_strcmp(pName, "draw:frame")) {
+        
         if (!UT_strcmp(m_rElementStack.getStartTag(0)->getName(), "text:p")) {
             // A paragraph anchored frame.
             // It's postponed because AbiWord uses paragraph frames *after* the
@@ -333,7 +339,16 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
             UT_ASSERT(UT_SHOULD_NOT_HAPPEN);
             // TODO: Figure out what to do then.
         }
+        
+    } else if (!UT_strcmp(pName, "draw:text-box")) {
+        UT_ASSERT(m_elementParsingLevel == 0);
+        
+        // We're inside a text-box, parsing its text contents.
+        m_bInSection = true;
+        m_bOnContentStream = true;
     }
+    
+    m_elementParsingLevel++;
 }
 
 
@@ -345,6 +360,8 @@ void OD_TextContent_ListenerState::startElement (const XML_Char* pName,
 void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
                                                OD_ListenerStateAction& rAction)
 {
+    UT_ASSERT(m_elementParsingLevel >= 0);
+    
     if (!UT_strcmp(pName, "text:section" )) {
 
         m_bInSection = false;
@@ -406,7 +423,8 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
         m_bAcceptingText = true;
         
     } else if (!UT_strcmp(pName, "office:text")) {
-        
+
+        UT_ASSERT(m_elementParsingLevel == 1);        
         UT_ASSERT(m_bOnContentStream);
         
         // We were inside a <office:text> element.
@@ -419,11 +437,11 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
                !UT_strcmp(pName, "style:footer") ||
                !UT_strcmp(pName, "style:header-left") ||
                !UT_strcmp(pName, "style:footer-left")) {
-                
-        // We were inside a <style:header/footer> element.
-        
+
+        UT_ASSERT(m_elementParsingLevel == 1);
         UT_ASSERT(!m_bOnContentStream);
         
+        // We were inside a <style:header/footer> element.
         rAction.popState();
         
     } else if (!UT_strcmp(pName, "text:list")) {
@@ -432,7 +450,14 @@ void OD_TextContent_ListenerState::endElement (const XML_Char* pName,
             m_pCurrentListStyle = NULL;
         }
         
+    } else if (!UT_strcmp(pName, "draw:text-box")) {
+        UT_ASSERT(m_elementParsingLevel == 1);
+        
+        // We were inside a <draw:text-box> element.
+        rAction.popState();
     }
+    
+    m_elementParsingLevel--;
 }
 
 
@@ -583,7 +608,7 @@ bool OD_TextContent_ListenerState::_pushInlineFmt(const XML_Char ** atts)
             return false;
         }
         
-        if (!m_stackFmtStartIndex.push((void*)start)) {
+        if (!m_stackFmtStartIndex.push(start)) {
             return false;
         }
     }
@@ -597,9 +622,9 @@ bool OD_TextContent_ListenerState::_pushInlineFmt(const XML_Char ** atts)
  */
 void OD_TextContent_ListenerState::_popInlineFmt(void)
 {
-    UT_uint32 start;
+    UT_sint32 start;
     
-    if (!m_stackFmtStartIndex.pop((void **)&start))
+    if (!m_stackFmtStartIndex.pop(&start))
         return;
         
     UT_uint32 k;
