@@ -89,7 +89,9 @@ private:
     bool m_bAcceptingText;
 
     UT_GenericVector<XML_Char*> m_vecInlineFmt;
-    UT_Stack m_stackFmtStartIndex;
+    UT_NumberStack m_stackFmtStartIndex;
+    
+    UT_sint8 m_elementParsingLevel;
 
 	// Buffer that stores character data defined between start and end element
 	// tags. e.g.: <bla>some char data</bla>
@@ -97,6 +99,7 @@ private:
 
     bool m_bInTOC;
 
+    // Table parsing stuff
     int m_row;
     int m_col;
     int m_cel;
